/*

  This is a part of the LiteStep Shell Source code.

  Copyright (C) 1997-2001 The LiteStep Development Team

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/****************************************************************************
01/28/00 - Charles Oliver Nutter (Headius)
  - Modified boolean statements to perform comparisons rather than forced
	  conversion
	- Added pragma to disable stl warnings
01/26/00 - Joachim Calvert (NeXTer)
  - The topmost problem is finally fixed with a little assistance from jugg
12/15/00 - Joachim Calvert (NeXTer)
  - Hopefully the ontop problems are gone
12/10/00 - Joachim Calvert (NeXTer)
  - Code cleanup in preparation for LS coding standards draft
11/30/00 - Joachim Calvert (NeXTer)
  - Cleaned up the new code
11/30/00 - Bobby G. Vinyard (Message)
  - Added code to move a shortcut whose position was not on a display monitor
    to the nearest monitor (This should cure the problem with shortcuts not
    being visible because of different resolution in multimon systems)
11/30/00 - Joachim Calvert (NeXTer)
  - Added the 'r' flag that makes the shortcut relative to the one before it
11/28/00 - Bobby G. Vinyard (Message)
  - Added multimonitor support
11/21/00 - Joachim Calvert (NeXTer)
  - Added the ability to place shortcuts relative the screen center. Just add
    a 'c' after the relevant coordinate (i.e., -42c)
10/06/00 - Joachim Calvert (NeXTer)
  - The caption set for the shortcut is now set as the window caption for
    the shortcut, so that operations can be performed on individual shortcuts
    when using an advanced scripting module
  - Added the ability to set whether or not captions are shown for all and
    individual shortcuts. See shortcut2.txt for details
08/20/00 - Joachim Calvert (NeXTer)
  - Added the ability to define default sounds, see shortcut2.txt for details
08/17/00 - Joachim Calvert (NeXTer)
  - Can be refreshed with the use of the !Refresh command
  - The shortcuts now revert to the normal state before being shown
08/15/00 - Joachim Calvert (NeXTer)
  - When using sounds you can now use .none for the mouseover to only get sound
    on click
08/10/00 - Joachim Calvert (NeXTer)
  - Added drag'n'drop, courtesy of Sylvain Rouquette (Syl)
08/07/00 - Joachim Calvert (NeXTer)
  - Added sound support, courtesy of Sylvain Rouquette (Syl)
05/28/00 - Joachim Calvert (NeXTer)
  - When using opaque shortcut images, you can speed up loading considerably
    by adding an 'o' to the flags. This will prevent the use of regions for
    the marked shortcut.
05/25/00 - Joachim Calvert (NeXTer)
  - After a tip from blkhawk, all drawing is now done in the WM_ERASEBKGND
    handler. Should be a wee bit smother than before.
05/21/00 - Joachim Calvert (NeXTer)
  - Had forgotten to reset the region after changing the topmost attribute.
05/20/00 - Joachim Calvert (NeXTer)
  - !ShortcutGroupOnTopToggle now works... I had forgotten to add the actual code
05/17/00 - Joachim Calvert (NeXTer)
  - Added the command !ShortcutGroupOnTopToggle.
05/12/00 - Joachim Calvert (NeXTer)
  - If a shortcut is started as hidden, the resources for it aren't created
    until it is shown, substantially decreasing the time to load LS if there
    are many hidden shortcuts.
04/27/00 - Joachim Calvert (NeXTer)
  - You now have to preceed any creation flags with '#', otherwise it'll be
    assumed that it's the command.
04/26/00 - Joachim Calvert (NeXTer)
  - After som thorough testing by rootrider, it should now be pretty stable.
    Currently supported bang commands are !ShortcutGroupToggle,
    !ShortcutGroupShow, !ShortcutGroupHide, !ShortcutGroupOnTop &
    !ShortcutGroupOnBottom, followed by one or more group numbers, or * to
    perform the operation on all groups.
04/25/00 - Joachim Calvert (NeXTer)
  - Complete rewrite from scratch, 2nd edition. The first edition
    accidentally got overwritten when I had finished it...
****************************************************************************/
#pragma warning(disable: 4786)
#include <stdio.h>
#include <windows.h>
#include <commctrl.h>

#include "debug.h"
#include "shortcut.h"

#include "lsapi.h"
#include "safestr.h"
#include "common.h"
#include "lswinbase.h"

using namespace std;

//---------------------------------------------------------
// Global defs and vars
//---------------------------------------------------------

const char APP_NAME[] = "ShortcutClass"; // Our window class, etc
const char RCS_REVISION[] = "$Revision: 1.72 $"; // Our Version
const char RCS_ID[] = "$Id: layercut - ver 1.72 (Syl) Exp $"; // The Full RCS ID.
const LEAVE_TIMER_ID		= 1;
const TIMER_FADE				= 2;
const TIMER_ANIM				= 3;
const UM_SETTOPMOST = WM_USER;

ShortcutFactory* shortcutFactory;

lpfnUpdateLayeredWindow UpdateLayeredWindow = NULL;
bool alpha_enable = false;
int default_alpha = 255;
BYTE alpha_threshold = 0;
bool loadingOnStartup = true;
bool noPosOnDisplayChange = false;
bool PosUpdate = false;
bool SoundSync = false;
bool PrimaryCoords = false;
bool BoundToMonitor = false;
t_range_mono fadeShow, fadeHide, fadeHoverIn, fadeHoverOut, fadeClickIn, fadeClickOut;
char buffer[MAX_LINE_LENGTH];

//---------------------------------------------------------
// Module startup and shutdown
//---------------------------------------------------------

int initModuleEx(HWND parentWnd, HINSTANCE hInst, LPCSTR)
{
  int code;

  //DebugSetFileName("c:\\log.txt");

  Window::init(hInst);

	OSVERSIONINFO os = { sizeof(os) };
	GetVersionEx(&os);
	alpha_enable = ( VER_PLATFORM_WIN32_NT == os.dwPlatformId && os.dwMajorVersion >= 5 );

	if (alpha_enable)
	{
		HMODULE hUser32 = GetModuleHandle(_T("USER32.DLL"));
		UpdateLayeredWindow = (lpfnUpdateLayeredWindow)GetProcAddress(hUser32, "UpdateLayeredWindow");
		alpha_enable = UpdateLayeredWindow != NULL;
	}

  shortcutFactory = new ShortcutFactory(parentWnd, code);

  return code;
}


void quitModule(HINSTANCE)
{
  delete shortcutFactory;
}


//=========================================================
// ShortcutFactory ////////////////////////////////////////
//=========================================================

ShortcutFactory::ShortcutFactory(HWND parentWnd, int& code):
Window(APP_NAME),
fSoundClick(NULL),
fSoundHover(NULL),
fShowCaptions(true),
fBangLock(false)
{
  fShortcutHints = CreateWindow(TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP,
                     CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
                     NULL, NULL, hInstance, NULL);
  if (fShortcutHints)
    SetWindowPos(fShortcutHints, HWND_TOPMOST, 0, 0, 0, 0,
                 SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);

  createWindow(WS_EX_TOOLWINDOW | WS_EX_NOACTIVATE, "ShortcutFactory", WS_POPUP,
               0, 0, 0, 0, parentWnd);
  SetWindowLong(hWnd, GWL_USERDATA, magicDWord);

	loadAnimation();
	loadExec();
  createShortcuts();
}


ShortcutFactory::~ShortcutFactory()
{
  destroyWindow();
}


void ShortcutFactory::relayHintMessage(HWND hWnd, Message& message)
{
  MSG hintMessage = {hWnd, message.uMsg, message.wParam, message.lParam};

  if (!fShortcutHints)
    return;
  SendMessage(fShortcutHints, TTM_RELAYEVENT, 0, LPARAM(&hintMessage));
  SetWindowPos(fShortcutHints, HWND_TOPMOST, 0, 0, 0, 0,
               SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
}


void ShortcutFactory::addHint(HWND hWnd, LPSTR caption)
{
  TOOLINFO ti;
  RECT clientRect;

  if (!fShortcutHints)
    return;
  GetClientRect(hWnd, &clientRect);
  ti.cbSize = sizeof(TOOLINFO);
  ti.uFlags = TTF_SUBCLASS;
  ti.hwnd = hWnd;
  ti.uId = 0;
  ti.rect = clientRect;
  ti.hinst = hInstance;
  ti.lpszText = caption;
  ti.lParam = 0;
  SendMessage(fShortcutHints, TTM_ADDTOOL, 0, LPARAM(&ti));
}


void ShortcutFactory::removeHint(HWND hWnd)
{
  TOOLINFO ti;
  RECT emptyRect = {0, 0, 0, 0};

  if (!fShortcutHints)
    return;
  ti.cbSize = sizeof(TOOLINFO);
  ti.uFlags = 0;
  ti.hwnd = hWnd;
  ti.uId = 0;
  ti.rect = emptyRect;
  ti.hinst = hInstance;
  ti.lpszText = NULL;
  ti.lParam = 0;
  SendMessage(fShortcutHints, TTM_DELTOOL, 0, LPARAM(&ti));
}


void LsboxModuleHook(HWND caller, const char *bangCommandName, const char *arguments)
{
	shortcutFactory->ModuleHook(caller, arguments);
}


void ShortcutFactory::ModuleHook(HWND caller, LPCSTR args)
{
	char *handle = strrchr(args,' ');
	if (!handle) 
		return;
	*handle = 0;

	Lsbox box;
	box.hwnd = (HWND)atoi(handle+1);

	box.name = string(args);
	for (int i = 0; i < vector_box.size(); i++)
		if (vector_box[i].name.compare(box.name) == 0)
			break;

	if (i >= vector_box.size())
	{
		vector_box.push_back(box);
		createShortcuts(&box);
	}
}


void ShortcutFactory::createShortcuts(Lsbox *box)
{
  char configLine[MAX_LINE_LENGTH];
  FILE* step;
  DWORD groupNumber;
  ShortcutMap::iterator smIter;
  Shortcut* newShortcut;



  /*GetRCString("shortcutFadeShow", configLine, "0", MAX_LINE_LENGTH);
	fadeShow.initial = atoi(configLine);
	fadeShow.enable = fadeShow.initial != 0;

  GetRCString("shortcutFadeHide", configLine, "0", MAX_LINE_LENGTH);
	fadeHide.initial = atoi(configLine);
	fadeHide.enable = fadeHide.initial != 0;

  GetRCString("shortcutFadeHoverIn", configLine, "0", MAX_LINE_LENGTH);
	fadeHoverIn.initial = atoi(configLine);
	fadeHoverIn.enable = fadeHoverIn.initial != 0;

  GetRCString("shortcutFadeHoverOut", configLine, "0", MAX_LINE_LENGTH);
	fadeHoverOut.initial = atoi(configLine);
	fadeHoverOut.enable = fadeHoverOut.initial != 0;

  GetRCString("shortcutFadeClickIn", configLine, "0", MAX_LINE_LENGTH);
	fadeClickIn.initial = atoi(configLine);
	fadeClickIn.enable = fadeClickIn.initial != 0;

  GetRCString("shortcutFadeClickOut", configLine, "0", MAX_LINE_LENGTH);
	fadeClickOut.initial = atoi(configLine);
	fadeClickOut.enable = fadeClickOut.initial != 0;*/



  GetRCString("shortcutDefaultAlpha", configLine, "255", MAX_LINE_LENGTH);
	default_alpha = atoi(configLine);

  GetRCString("shortcutAlphaThreshold", configLine, "0", MAX_LINE_LENGTH);
	alpha_threshold = atoi(configLine);

  GetRCString("shortcutSoundClick", configLine, ".none", MAX_LINE_LENGTH);
  if (!StrIPos(configLine, ".none"))
  {
    fSoundClick = new char[StrLen(configLine) + 1];
    StrCopy(fSoundClick, configLine);
  }
  GetRCString("shortcutSoundHover", configLine, ".none", MAX_LINE_LENGTH);
  if (!StrIPos(configLine, ".none"))
  {
    fSoundHover = new char[StrLen(configLine) + 1];
    StrCopy(fSoundHover, configLine);
  }
  fShowCaptions = GetRCBoolDef("shortcutShowCaptions", TRUE) != FALSE;
  
  loadingOnStartup = GetRCBool("shortcutNoLoadingOnStartup", false);
  SoundSync = GetRCBool("ShortcutSoundSync", true);
  noPosOnDisplayChange = GetRCBool("ShortcutNoDisplayChange", true);
  PosUpdate = GetRCBool("ShortcutPosUpdate", true);
  PrimaryCoords = GetRCBool("ShortcutPrimaryMonitor", true);
  BoundToMonitor = GetRCBool("ShortcutBoundToMonitor", true);

  step = LCOpen(box ? box->name.c_str() : NULL);
  if (step)
  {
	  while (LCReadNextConfig(step, box ? "*ShortcutBox" : "*shortcut", configLine, MAX_LINE_LENGTH))
    {
      newShortcut = new Shortcut(this, configLine, groupNumber, box);
      smIter = fShortcutGroups.find(groupNumber);
      if (smIter == fShortcutGroups.end())
        fShortcutGroups.insert(ShortcutMap::value_type(groupNumber, newShortcut));
      else
      {
        newShortcut->append(smIter->second);
        smIter->second = newShortcut;
      }
    }
    LCClose(step);
  }
}


void ShortcutFactory::destroyShortcuts()
{
  ShortcutMap::iterator smIter;

  for (smIter = fShortcutGroups.begin(); smIter != fShortcutGroups.end(); smIter++)
    if (smIter->second)
      delete smIter->second;
  fShortcutGroups.clear();
	vector_box.clear();

  if (fSoundClick)
    delete[] fSoundClick;
  if (fSoundHover)
    delete[] fSoundHover;
  fSoundClick = NULL;
  fSoundHover = NULL;
}


void ShortcutFactory::setVisible(LPCSTR groups, int state)
{
  char group[MAX_LINE_LENGTH];
  LPCSTR nextGroup = groups;
  ShortcutMap::iterator smIter;

  if (CharInStr('*', groups))
  {
    for (smIter = fShortcutGroups.begin(); smIter != fShortcutGroups.end(); smIter++)
    {
      if (smIter->second)
        smIter->second->setVisible(state);
    }
  }
  else while (GetToken(nextGroup, group, &nextGroup, false))
  {
    smIter = fShortcutGroups.find(atoi(group));
    if (smIter != fShortcutGroups.end())
      if (smIter->second)
        smIter->second->setVisible(state);
  }
}

void ShortcutFactory::setAlpha(LPCSTR groups)
{
  char group[MAX_LINE_LENGTH];
  LPCSTR nextGroup = groups;
  ShortcutMap::iterator smIter;
	int val;

	GetToken(nextGroup, group, &nextGroup, false);
	val = atoi(group);


  if (CharInStr('*', nextGroup))
  {
    for (smIter = fShortcutGroups.begin(); smIter != fShortcutGroups.end(); smIter++)
    {
      if (smIter->second)
        smIter->second->setAlpha(val);
    }
  }
  else while (GetToken(nextGroup, group, &nextGroup, false))
  {
    smIter = fShortcutGroups.find(atoi(group));
    if (smIter != fShortcutGroups.end())
      if (smIter->second)
        smIter->second->setAlpha(val);
  }
}

//---------------------------------------------------------
// Bang commands
//---------------------------------------------------------

void ShortcutFactory::bangGroupToggle(HWND sender, LPCSTR args)
{
  shortcutFactory->setVisible(args, 0);
}


void ShortcutFactory::bangGroupShow(HWND sender, LPCSTR args)
{
  shortcutFactory->setVisible(args, 1);
}


void ShortcutFactory::bangGroupHide(HWND sender, LPCSTR args)
{
  shortcutFactory->setVisible(args, -1);
}


void ShortcutFactory::bangGroupOnTopToggle(HWND sender, LPCSTR args)
{
  int state = 0;
  while (shortcutFactory->fBangLock);
  shortcutFactory->fBangLock = true;
  SendMessage(shortcutFactory->handle(), UM_SETTOPMOST, WPARAM(args), LPARAM(state));
  shortcutFactory->fBangLock = false;
}


void ShortcutFactory::bangGroupOnTop(HWND sender, LPCSTR args)
{
  int state = 1;

  while (shortcutFactory->fBangLock);
  shortcutFactory->fBangLock = true;
  SendMessage(shortcutFactory->handle(), UM_SETTOPMOST, WPARAM(args), LPARAM(state));
  shortcutFactory->fBangLock = false;
}


void ShortcutFactory::bangGroupOnBottom(HWND sender, LPCSTR args)
{
  int state = -1;

  while (shortcutFactory->fBangLock);
  shortcutFactory->fBangLock = true;
  SendMessage(shortcutFactory->handle(), UM_SETTOPMOST, WPARAM(args), LPARAM(state));
  shortcutFactory->fBangLock = false;
}



void ShortcutFactory::bangGroupSetAlpha(HWND sender, LPCSTR args)
{
	shortcutFactory->setAlpha(args);
}

//---------------------------------------------------------
// Message handling
//---------------------------------------------------------

void ShortcutFactory::windowProc(Message& message)
{
  BEGIN_MESSAGEPROC
    MESSAGE(onCreate,     WM_CREATE)
    MESSAGE(onDestroy,    WM_DESTROY)
    MESSAGE(onGetRevId,   LM_GETREVID)
    MESSAGE(onRefresh,    LM_REFRESH)
    MESSAGE(onSetTopmost, UM_SETTOPMOST)
  END_MESSAGEPROC
}


void ShortcutFactory::onCreate(Message& message)
{
  int lsMessages[] = {LM_GETREVID, LM_REFRESH, 0};

  SendMessage(hParent, LM_REGISTERMESSAGE, WPARAM(hWnd), LPARAM(lsMessages));

  AddBangCommand("!shortcutgrouptoggle", bangGroupToggle);
  AddBangCommand("!shortcutgroupshow", bangGroupShow);
  AddBangCommand("!shortcutgrouphide", bangGroupHide);
  AddBangCommand("!shortcutgroupontoptoggle", bangGroupOnTopToggle);
  AddBangCommand("!shortcutgroupontop", bangGroupOnTop);
  AddBangCommand("!shortcutgrouponbottom", bangGroupOnBottom);
  AddBangCommand("!shortcutgroupsetalpha", bangGroupSetAlpha);
  AddBangCommandEx("!LayercutModuleHook", LsboxModuleHook);
}


void ShortcutFactory::onDestroy(Message& message)
{
  ShortcutMap::iterator smIter;
  int lsMessages[] = {LM_GETREVID, LM_REFRESH, 0};

  destroyShortcuts();
  
  if (fShortcutHints)
    DestroyWindow(fShortcutHints);

  SendMessage(hParent, LM_UNREGISTERMESSAGE, WPARAM(hWnd), LPARAM(lsMessages));

  RemoveBangCommand("!shortcutgrouptoggle");
  RemoveBangCommand("!shortcutgroupshow");
  RemoveBangCommand("!shortcutgrouphide");
  RemoveBangCommand("!shortcutgroupontoptoggle");
  RemoveBangCommand("!shortcutgroupontop");
  RemoveBangCommand("!shortcutgrouponbottom");
  RemoveBangCommand("!shortcutgroupsetalpha");
  RemoveBangCommand("!LayercutModuleHook");
}


void ShortcutFactory::onGetRevId(Message& message)
{
  char* buf = (char*)(message.lParam);

  switch (message.wParam)
  {
    case 0:
      sprintf(buf, "layercut.dll: %s", &RCS_REVISION[11]);
      buf[strlen(buf) - 1] = '\0';
      break;
    default:
      strcpy(buf, "");
  }
  message.lResult = strlen(buf);
}


void ShortcutFactory::onRefresh(Message& message)
{
  destroyShortcuts();
  createShortcuts();
}


void ShortcutFactory::onSetTopmost(Message& message)
{
  LPCSTR groups = LPCSTR(message.wParam);
  int topMost = int(message.lParam);
  char group[MAX_LINE_LENGTH];
  LPCSTR nextGroup = groups;
  ShortcutMap::iterator smIter;

  if (CharInStr('*', groups))
  {
    for (smIter = fShortcutGroups.begin(); smIter != fShortcutGroups.end(); smIter++)
      if (smIter->second)
        smIter->second->setTopMost(topMost);
  }
  else while (GetToken(nextGroup, group, &nextGroup, false))
  {
    smIter = fShortcutGroups.find(atoi(group));
    if (smIter != fShortcutGroups.end())
      if (smIter->second)
        smIter->second->setTopMost(topMost);
  }
}





//=========================================================
// Shortcut ///////////////////////////////////////////////
//=========================================================

Shortcut::Shortcut(ShortcutFactory* owner, LPCSTR szLine, DWORD& group, Lsbox *box):
Window(APP_NAME),
fOwner(owner),
fNext(NULL),
fCaption(NULL),
fId(NULL),
fCommand(NULL),
fCommandArgs(NULL),
fCommand2(NULL),
fCommand2Args(NULL),
fExec(NULL),
nextAnim(NULL),
countAnim(0),
lastPos(0),
img1(NULL),
img2(NULL),
fHoverSound(NULL),
fClickSound(NULL),
fState(ssNormal),
m_box(NULL),
//fChangeZOrder(false),
fIsTopMost(false),
fIsVisible(true),
fIsOpaque(false),
fShowCaption(true),
fExecHover(false),
fExecOut(false),
fLeaveTimer(0),
fFadeTimer(0),
fAnimTimer(0),
fWidth(32),
fHeight(32),
fPaintDC(NULL)
{
	if (box)
		m_box = new Lsbox(*box);

	alpha.initial = alpha.current = default_alpha;
	alpha.enable = false;
  group = parseConfig(szLine);

  if (loadingOnStartup || fIsVisible)
		createShortcut();
}


Shortcut::~Shortcut()
{
  if (fNext)
    delete fNext;
  clear();
  if (m_box)
	  delete m_box;
  destroyWindow();
}


void Shortcut::append(Shortcut* shortcut)
{
  fNext = shortcut;
}


void Shortcut::setVisible(int visible)
{
  bool wasVisible = fIsVisible;

  if (!visible)
    fIsVisible = !fIsVisible;
  else
    fIsVisible = visible > 0;

  if (!hWnd && fIsVisible)
    createShortcut();

  if (fIsVisible != wasVisible)
	{
		lastPos = 0;
		countAnim = 0;
		setImage(fIsVisible ? fNormalImage : NULL);
		SetWindowPos(handle(), NULL, 0, 0, 0, 0,
		SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER |
		(fIsVisible ? SWP_SHOWWINDOW : SWP_HIDEWINDOW));/**/
	}

  if (fNext)
    fNext->setVisible(visible);
}

void Shortcut::setTopMost(int topMost)
{
	bool wasTopMost = fIsTopMost;

//	fChangeZOrder = true;
	if (!topMost)
		fIsTopMost = !fIsTopMost;
	else
		fIsTopMost = topMost > 0;

	if (fNext && fIsTopMost)
		fNext->setTopMost(topMost);

	if (hWnd && (fIsTopMost != wasTopMost))
	{
		zOrder(hWnd, m_box, fIsTopMost);
	}

	if (fNext && !fIsTopMost)
		fNext->setTopMost(topMost);
//	fChangeZOrder = false;
}


void Shortcut::setAlpha(int val)
{
	alpha.initial = val;
	updateAlpha(val);

  if (fNext)
    fNext->setAlpha(val);
}


int get_val(char *str)
{
	int ret;

	ret = 0;
	do
	{
		ret = ret * 10 + (*str - '0');
		str++;
	} while (*str);
	return ret;
}

DWORD Shortcut::parseConfig(LPCSTR szConfigLine)
{
  char token[MAX_LINE_LENGTH];
  LPCSTR nextToken = szConfigLine;
  DWORD result = 0;
  bool hasCommand = true;
  bool clickSoundDef = false;
  bool hoverSoundDef = false;
  static coord lastLeft;
  static coord lastTop;

  clear();

  GetToken(nextToken, NULL, &nextToken, false);
  if (GetToken(nextToken, token, &nextToken, false))
  {
    fCaption = new char[StrLen(token) + 1];
    StrCopy(fCaption, token);
	if (nextToken && nextToken[0] == ':')
	{
		if ((++nextToken)[0] == ' ')
			StrCopy(token, "");
		else
			GetToken(nextToken, token, &nextToken, false);

		fId = new char[StrLen(token) + 1];
		StrCopy(fId, token);
	}
  }

  if (GetToken(nextToken, token, &nextToken, false))
  {
	fLeft.absolute = (*token == '~');
	if (fLeft.absolute)
		token[0] = '-';
    fLeft.n = atoi(token);
    strlwr(token);
    if (strchr(token, 'r'))
	{
      lastLeft.n += fLeft.n;
      fLeft = lastLeft;
	  if (!fLeft.absolute)
		fLeft.absolute = !lastLeft.negative;
	}
	else
	{
      fLeft.center = strchr(token, 'c') != 0;
      fLeft.primary = strchr(token, 'p') != 0;
	  fLeft.negative = !fLeft.absolute && (*token == '-');
	}
  }
  lastLeft = fLeft;
  if (GetToken(nextToken, token, &nextToken, false))
  {
	fTop.absolute = (*token == '~');
	if (fTop.absolute)
		token[0] = '-';
    fTop.n = atoi(token);
    strlwr(token);
    if (strchr(token, 'r'))
	{
      lastTop.n += fTop.n;
      fTop = lastTop;
	  if (!fTop.absolute)
		fTop.absolute = !lastTop.negative;
	}
	else
	{
      fTop.center = strchr(token, 'c') != 0;
      fTop.primary = strchr(token, 'p') != 0;
	  fTop.negative = !fTop.absolute && (*token == '-');
	}
  }
  lastTop = fTop;

  if (GetToken(nextToken, token, &nextToken, false))
    fNormalName = string(token);
  if (GetToken(nextToken, token, &nextToken, false))
    fHoverName = string(token);
  if (GetToken(nextToken, token, &nextToken, false))
    fClickName = string(token);

  if (GetToken(nextToken, token, &nextToken, false))
  {
    if (token[0] == '#')
    {
      strlwr(token);
      result = atoi(&token[1]);
      fIsVisible = !strchr(token, 'h');
      fIsTopMost = strchr(token, 't')  != 0;
      fIsOpaque = strchr(token, 'o')  != 0;
	  fIsTransparent = CharInStr('k', token) || CharInStr('K', token);
			fExecHover = CharInStr('e', token) || CharInStr('E', token);
			fExecOut = CharInStr('s', token) || CharInStr('S', token);
      if (fOwner->getShowCaptions())
        fShowCaption = true; //!strchr(token, 'c');
      else
        fShowCaption = strchr(token, 'c')  != 0;
      hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
    }

    if (StrIPos(token, "a:") == token)
    {
			alpha.initial = alpha.current = get_val(token + 2);
			hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
		}

    if (StrIPos(token, ".wav"))
    {
      fHoverSound = new char[StrLen(token) + 1];
      StrCopy(fHoverSound, token);
      hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
    }
    else if (StrIPos(token, ".none"))
      hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
    else if (StrIPos(token, ".def"))
    {
      hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
      hoverSoundDef = true;
    }
    else
      hoverSoundDef = true;

    if (StrIPos(token, ".wav"))
    {
      fClickSound = new char[StrLen(token) + 1];
      StrCopy(fClickSound, token);
      hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
    }
    else if (StrIPos(token, ".none"))
      hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
    else if (StrIPos(token, ".def"))
    {
      hasCommand = GetToken(nextToken, token, &nextToken, false) != FALSE;
      clickSoundDef = true;
    }
    else
      clickSoundDef = true;

    if (hoverSoundDef && fOwner->getSoundHover())
    {
      fHoverSound = new char[StrLen(fOwner->getSoundHover()) + 1];
      StrCopy(fHoverSound, fOwner->getSoundHover());
    }

    if (clickSoundDef && fOwner->getSoundClick())
    {
      fClickSound = new char[StrLen(fOwner->getSoundClick()) + 1];
      StrCopy(fClickSound, fOwner->getSoundClick());
    }

    if (hasCommand)
    {
      fCommand = new char[StrLen(token) + 1];
      StrCopy(fCommand, token);
			fExec = fOwner->getExec(string(fCommand));
    }
    if (nextToken)
    {
			LPCSTR tk = StrIPos(strlwr((char *)nextToken), "!execute");
			if (tk && (strnicmp(nextToken, "!execute", 8) != 0))
			{
				fCommandArgs = new char[tk - nextToken];
				StrLCopy(fCommandArgs, nextToken, tk - nextToken - 1);

				fCommand2 = new char[strlen("!execute") + 1];
				fCommand2 = strdup("!execute");

				fCommand2Args = new char[StrLen(tk + strlen("!execute") + 1) + 1];
				StrCopy(fCommand2Args, tk + strlen("!execute") + 1);
			}
			else
			{
				fCommandArgs = new char[StrLen(nextToken) + 1];
				StrCopy(fCommandArgs, nextToken);
			}
    }
  }

  return result;
}


void Shortcut::createShortcut()
{
	CAnimation anim;

	anim.is_opaque = fIsOpaque;

  if (!fNormalName.empty())
    fNormalImage = fOwner->loadImage(fNormalName, &anim);
  if (!fHoverName.empty())
		fHoverImage = fOwner->loadImage(fHoverName, &anim);
  if (!fClickName.empty())
		fClickImage = fOwner->loadImage(fClickName, &anim);

  fRelOrigin = convertCoords(fLeft, fTop);

  createWindow(WS_EX_TOOLWINDOW | WS_EX_NOACTIVATE |
		(m_box ? 0 : WS_EX_LAYERED) |
	  (fIsTransparent ? WS_EX_TRANSPARENT : 0),
			fId ? fId : fCaption,
			(m_box ? (WS_CHILD | WS_CLIPSIBLINGS) : WS_POPUP),
			fRelOrigin.x, fRelOrigin.y, fWidth, fHeight,
			(m_box ? m_box->hwnd : GetLitestepWnd()));

	if (m_box)
	{
		Message temp;
		onCreate(temp);
	}

//	fChangeZOrder = true;
	zOrder(hWnd, m_box, fIsTopMost);

	if (fIsVisible)
	{
		SetWindowPos(handle(), NULL, 0, 0, 0, 0,
			SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER |
			SWP_SHOWWINDOW);
	}
}


void Shortcut::clear()
{
  if (fCaption)
    delete[] fCaption;
  if (fId)
    delete[] fId;
  if (fCommand)
    delete[] fCommand;
  if (fCommandArgs)
    delete[] fCommandArgs;
  if (fCommand2)
    delete[] fCommand2;
  if (fCommand2Args)
    delete[] fCommand2Args;
  if (fHoverSound)
    delete[] fHoverSound;
  if (fClickSound)
    delete[] fClickSound;

  if (fPaintDC)
    DeleteDC(fPaintDC);

  fCaption = NULL;
  fId = NULL;
  fCommand = NULL;
  fCommandArgs = NULL;
  fCommand2 = NULL;
  fCommand2Args = NULL;
  fHoverSound = NULL;
  fClickSound = NULL;
  fPaintDC = NULL;

	fNormalName.erase();
	fHoverName.erase();
	fClickName.erase();
	fNormalImage = NULL;
	fHoverImage = NULL;
	fClickImage = NULL;
}


void Shortcut::setImage(CAnimation *anim, bool reset)
{
	bool set_anim = true;
	RECT clientRect;

  if (!anim)
    return;

	WINDOWPLACEMENT win_pos;
	GetWindowPlacement(hWnd, &win_pos);
	fRelOrigin.x = win_pos.rcNormalPosition.left;
	fRelOrigin.y = win_pos.rcNormalPosition.top;

	if (!nextAnim)
	{
		nextAnim = anim;
		img1 = anim;
	}
	if (!reset)
		nextAnim = anim;

	if (!img1->is_animated)
		img1 = nextAnim;
	else if ((nextAnim != anim) && ((nextAnim->priority) || (!img1->finish) || (lastPos >= img1->max_size - 1)))
	{
		set_anim = true;
		countAnim = 0;
		img1->pos = 0;
		lastPos = 0;
		if (fAnimTimer)
		{
			KillTimer(hWnd, fAnimTimer);
			fAnimTimer = 0;
		}
		img1 = nextAnim;
	}
	else
		if (fAnimTimer)
			set_anim = false;

	if ((!img1->loop) && (lastPos >= img1->max_size))
	{
		set_anim = (img1 != nextAnim);
		if (fAnimTimer)
		{
			KillTimer(hWnd, fAnimTimer);
			fAnimTimer = 0;
		}
	}

	if (!img1)
		return;
	fIsOpaque = img1->is_opaque;
	img1->setPos(countAnim, (nextAnim != anim));
	countAnim++;
	lastPos = img1->pos;
	fWidth = img1->width;
	fHeight = img1->height;
  SetWindowPos(hWnd, NULL, fRelOrigin.x, fRelOrigin.y, fWidth, fHeight, SWP_NOACTIVATE | SWP_NOZORDER);

	if (set_anim && (img1->is_animated))
	{
		//DebugPrint("set timer %s (%d)\n", img1->name.c_str(), img1->speed);
		fAnimTimer = SetTimer(hWnd, TIMER_ANIM, img1->speed ? img1->speed : 100, NULL);
		if (fIsOpaque)
		{
			HRGN windowRgn = CreateRectRgn(0, 0, fWidth, fHeight);
			CombineRgn(windowRgn, img1->current->region, NULL, RGN_COPY);
			SetWindowRgn(hWnd, windowRgn, true);
		}
	}

  if (!fIsOpaque)
  {
    HRGN windowRgn = CreateRectRgn(0, 0, 0, 0);
    CombineRgn(windowRgn, img1->current->region, NULL, RGN_COPY);
    SetWindowRgn(hWnd, windowRgn, true);
  }

	GetClientRect(hWnd, &clientRect);
	InvalidateRect(hWnd, &clientRect, true);

	if (!m_box)
		updateAlpha(alpha.current, false);
}



//---------------------------------------------------------
// Message handling
//---------------------------------------------------------

void Shortcut::windowProc(Message& message)
{
  BEGIN_MESSAGEPROC
    MESSAGE(onCreate,            WM_CREATE)
    MESSAGE(onDestroy,           WM_DESTROY)
    MESSAGE(onEndSession,        WM_ENDSESSION)
    MESSAGE(onEndSession,        WM_QUERYENDSESSION)
    MESSAGE(onSysCommand,        WM_SYSCOMMAND)
    MESSAGE(onMouseMove,         WM_MOUSEMOVE)
    MESSAGE(onMouseLeave,        WM_MOUSELEAVE)
    MESSAGE(onMouseActivate,     WM_MOUSEACTIVATE)
    MESSAGE(onMove,              WM_MOVE)
    MESSAGE(onTimer,             WM_TIMER)
    MESSAGE(onButtonDown,        WM_LBUTTONDOWN)
    MESSAGE(onLButtonUp,         WM_LBUTTONUP)
    MESSAGE(onButtonDown,        WM_RBUTTONDOWN)
    MESSAGE(onRButtonUp,         WM_RBUTTONUP)
    MESSAGE(onButtonDown,        WM_MBUTTONDOWN)
    MESSAGE(onMButtonUp,         WM_MBUTTONUP)
    MESSAGE(onDropFiles,         WM_DROPFILES)
    MESSAGE(onEraseBkgnd,        WM_ERASEBKGND)
	//MESSAGE(onEraseBkgnd,        WM_PAINT)
    MESSAGE(onDisplayChange,     WM_DISPLAYCHANGE)
    MESSAGE(onShowWindow,        WM_SHOWWINDOW)
    MESSAGE(onUpdateAlpha,       WM_ALPHA)
	//MESSAGE(onUpdatePos,		 WM_WINDOWPOSCHANGING)
  END_MESSAGEPROC
}


void Shortcut::onCreate(Message& message)
{
  DragAcceptFiles(hWnd, true);
  SetWindowLong(hWnd, GWL_USERDATA, magicDWord);
  setImage(fIsVisible ? fNormalImage : NULL);
  if (fCaption && fShowCaption)
    fOwner->addHint(hWnd, fCaption);
}


void Shortcut::onDestroy(Message& message)
{
//	if (m_box)
//		SetParent(handle(), NULL);

  if (fLeaveTimer)
    KillTimer(hWnd, fLeaveTimer);
  fLeaveTimer = 0;
  if (fFadeTimer)
    KillTimer(hWnd, fFadeTimer);
  fFadeTimer = 0;
  if (fAnimTimer)
    KillTimer(hWnd, fAnimTimer);
  fAnimTimer = 0;
  fOwner->removeHint(hWnd);
}


void Shortcut::onEndSession(Message& message)
{
  message.lResult = SendMessage(GetLitestepWnd(), message.uMsg, message.wParam, message.lParam);
}


void Shortcut::onSysCommand(Message& message)
{
  if (message.wParam == SC_CLOSE)
    PostMessage(GetLitestepWnd(), WM_KEYDOWN, LM_SHUTDOWN, 0);
  else
    message.lResult = DefWindowProc(hWnd, message.uMsg, message.wParam, message.lParam);
}


void Shortcut::onMouseMove(Message& message)
{
	/*TRACKMOUSEEVENT EventTrack;
	EventTrack.cbSize = sizeof(TRACKMOUSEEVENT);
	EventTrack.dwFlags = TME_LEAVE;
	EventTrack.hwndTrack = hWnd;
	TrackMouseEvent(&EventTrack);/**/
  if (!fLeaveTimer)
    fLeaveTimer = SetTimer(hWnd, LEAVE_TIMER_ID, 20, NULL);

  if ((fState != ssNormal) && (fState != ssNormalAfterClicked))
    return;

  setImage(fHoverImage);
  
  if (fHoverSound)
    PlaySound(fHoverSound, NULL, SND_FILENAME | SND_ASYNC | SND_NOWAIT);

  fOwner->relayHintMessage(hWnd, message);

  if (fState != ssNormalAfterClicked)
  {
		if (fExec && fExec->b_in)
		{
			if (!fExec->in_arg.empty())
				sprintf(buffer, "[%s] %s", fExec->in.c_str(), fExec->in_arg.c_str());
			else
				sprintf(buffer, "[%s]", fExec->in.c_str());
			LSExecute(hWnd, buffer, SW_SHOWDEFAULT);
		}
		else if (fExecHover && fCommand2)
		{
			if (fCommand2Args)
				sprintf(buffer, "[%s] %s", fCommand2, fCommand2Args);
			else
				sprintf(buffer, "[%s]", fCommand2);
			LSExecute(hWnd, buffer, SW_SHOWDEFAULT);
		}
  }
  fState = ssHover;
}


void Shortcut::onMouseActivate(Message& message)
{
  //PostMessage(hWnd, WM_LBUTTONDOWN, 0, 0);
  message.lResult = MA_NOACTIVATE;
}


void Shortcut::onTimer(Message& message)
{
  switch (message.wParam) {
	case LEAVE_TIMER_ID:
		timerHover();
    break;
	case TIMER_FADE:
		timerFade();
		break;
	case TIMER_ANIM:
		timerAnim();
		break;
	}
}

void Shortcut::onMouseLeave(Message& message)
{
	if (fExec && fExec->b_out)
	{
		if (!fExec->out_arg.empty())
			sprintf(buffer, "[%s] %s", fExec->out.c_str(), fExec->out_arg.c_str());
		else
			sprintf(buffer, "[%s]", fExec->out.c_str());
		LSExecute(hWnd, buffer, SW_SHOWDEFAULT);
	}
	else if (fExecOut && fCommand2)
	{
		if (fCommand2Args)
			sprintf(buffer, "[%s] %s", fCommand2, fCommand2Args);
		else
			sprintf(buffer, "[%s]", fCommand2);
		LSExecute(hWnd, buffer, SW_SHOWDEFAULT);
	}/**/

  if (fState != ssNormal)
  {
    fState = ssNormal;
    setImage(fNormalImage);
  }
}

void Shortcut::timerHover()
{
  POINT cursorPos;
  bool isWithin;

  GetCursorPos(&cursorPos);

	if (m_box)
	{
		WINDOWPLACEMENT win_pos;
		GetWindowPlacement(GetParent(hWnd), &win_pos);
		cursorPos.x -= fRelOrigin.x + win_pos.rcNormalPosition.left;
		cursorPos.y -= fRelOrigin.y + win_pos.rcNormalPosition.top;
	}
	else
	{
		cursorPos.x -= fRelOrigin.x;
		cursorPos.y -= fRelOrigin.y;
	}

  if (fIsOpaque)
  {
    RECT clientRect;

    GetClientRect(hWnd, &clientRect);
    isWithin = PtInRect(&clientRect, cursorPos) != FALSE;
  }
  else
  {
    HRGN windowRgn = CreateRectRgn(0, 0, 0, 0);

    GetWindowRgn(hWnd, windowRgn);
    isWithin = PtInRegion(windowRgn, cursorPos.x, cursorPos.y) != FALSE;
    DeleteObject(windowRgn);
  }

  if (!isWithin)
  {
		if (fExec && fExec->b_out)
		{
			if (!fExec->out_arg.empty())
				sprintf(buffer, "[%s] %s", fExec->out.c_str(), fExec->out_arg.c_str());
			else
				sprintf(buffer, "[%s]", fExec->out.c_str());
			LSExecute(hWnd, buffer, SW_SHOWDEFAULT);
		}
		else if (fExecOut && fCommand2)
		{
			if (fCommand2Args)
				sprintf(buffer, "[%s] %s", fCommand2, fCommand2Args);
			else
				sprintf(buffer, "[%s]", fCommand2);
			LSExecute(hWnd, buffer, SW_SHOWDEFAULT);
		}/**/

    if (fState != ssNormal)
    {
      fState = ssNormal;
      setImage(fNormalImage);
    }

    KillTimer(hWnd, fLeaveTimer);
    fLeaveTimer = 0;
  }
}

void Shortcut::timerFade()
{}

void Shortcut::timerAnim()
{
	setImage(img1, true);
	lastPos = img1->pos;
  if (!img1->loop && (lastPos >= img1->max_size))
	{
		KillTimer(hWnd, fAnimTimer);
		fAnimTimer = 0;
	}
}

void Shortcut::onButtonDown(Message& message)
{
  if (fState == ssClick)
    return;

  fState = ssClick;

  if (fClickSound)
	  PlaySound(fClickSound, NULL, SND_FILENAME | (SoundSync ? SND_SYNC : SND_ASYNC) | SND_NOWAIT);

  setImage(fClickImage);
}


void Shortcut::onLButtonUp(Message& message)
{
  if (fState != ssClick)
    return;

  fState = (fExec && fExec->b_in) ? ssNormalAfterClicked : ssNormal;
  setImage(fNormalImage);

  if (fExec && fExec->b_left)
  {
    if (!fExec->left_arg.empty())
      sprintf(buffer, "[%s] %s", fExec->left.c_str(), fExec->left_arg.c_str());
    else
      sprintf(buffer, "[%s]", fExec->left.c_str());
    LSExecute(hWnd, buffer, SW_SHOWDEFAULT);
  }
  else if (fCommand)
  {
    if (fCommandArgs)
      sprintf(buffer, "[%s] %s", fCommand, fCommandArgs);
    else
      sprintf(buffer, "[%s]", fCommand);
    LSExecute(hWnd, buffer, SW_SHOWDEFAULT);
  }
}

void Shortcut::onRButtonUp(Message& message)
{
  if (fState != ssClick)
    return;

  fState = (fExec && fExec->b_in) ? ssNormalAfterClicked : ssNormal;
  setImage(((!fHoverImage) ? fNormalImage : fHoverImage));

  if (fExec && fExec->b_right)
  {
    if (!fExec->right_arg.empty())
      sprintf(buffer, "[%s] %s", fExec->right.c_str(), fExec->right_arg.c_str());
    else
      sprintf(buffer, "[%s]", fExec->right.c_str());
    LSExecute(hWnd, buffer, SW_SHOWDEFAULT);
  }
  else if (fCommand2)
  {
    if (fCommand2Args)
      sprintf(buffer, "[%s] %s", fCommand2, fCommand2Args);
    else
      sprintf(buffer, "[%s]", fCommand2);
    LSExecute(hWnd, buffer, SW_SHOWDEFAULT);
  }
}

void Shortcut::onMButtonUp(Message& message)
{
  if (fState != ssClick)
    return;

  fState = (fExec && fExec->b_in) ? ssNormalAfterClicked : ssNormal;
  setImage(((!fHoverImage) ? fNormalImage : fHoverImage));

  if (fExec && fExec->b_middle)
  {
    if (!fExec->middle_arg.empty())
      sprintf(buffer, "[%s] %s", fExec->middle.c_str(), fExec->middle_arg.c_str());
    else
      sprintf(buffer, "[%s]", fExec->middle.c_str());
    LSExecute(hWnd, buffer, SW_SHOWDEFAULT);
  }
}

void Shortcut::onDropFiles(Message& message)
{
  UINT fileCount = DragQueryFile(HDROP(message.wParam), ~0, NULL, 0);

  if (!fileCount)
    return;
  
  LPTSTR iter;

  if (fExec && fExec->b_drop)
  {
    if (!fExec->drop_arg.empty())
      sprintf(buffer, "[%s] %s", fExec->drop.c_str(), fExec->drop_arg.c_str());
    else
      sprintf(buffer, "[%s]", fExec->drop.c_str());
  }
  else if (!fExec && fCommand)
    sprintf(buffer, "[%s]", fCommand);
  else
	return;

  if (fClickSound)
    PlaySound(fClickSound, NULL, SND_FILENAME | SND_ASYNC | SND_NOWAIT);

  iter = buffer + StrLen(buffer);

  for (int i = 0; i < fileCount; i++)
  {
    strcpy(iter, " \"");
    iter += StrLen(iter);
    iter += DragQueryFile(HDROP(message.wParam), i, iter, 
                          MAX_LINE_LENGTH - (iter - buffer));
    *iter++ = '\"';
  }
  *iter = '\0';

  LSExecute(hWnd, buffer, SW_SHOWDEFAULT);
}


void Shortcut::onEraseBkgnd(Message& message)
{
  HDC hDC = (HDC)message.wParam;
  HBITMAP oldImage;

  if (!img1)
    return;
  if (!fPaintDC)
    fPaintDC = CreateCompatibleDC(hDC);
  oldImage = HBITMAP(SelectObject(fPaintDC, img1->current->image));
  BitBlt(hDC, 0, 0, fWidth, fHeight, fPaintDC, 0, 0, SRCCOPY);
  SelectObject(fPaintDC, oldImage);
}


void Shortcut::onDisplayChange(Message& message)
{
	if (noPosOnDisplayChange)
		return;

  fRelOrigin = convertCoords(fLeft, fTop);
  SetWindowPos(hWnd, NULL, fRelOrigin.x, fRelOrigin.y, 0, 0,
		SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER);
}


void Shortcut::onShowWindow(Message& message)
{
  if (message.wParam)
    setImage(fNormalImage);
}



void Shortcut::onMove(Message& message)
{
	if (!PosUpdate)
		return;
	
	POINT ptNew = { sLOWORD(message.lParam), sHIWORD(message.lParam) };
	POINT pt = convertCoords(fLeft, fTop);

	if (ptNew.x != pt.x)
	{
		int left = PrimaryCoords != fLeft.primary ? 0 : SCREEN_LEFT;
		int width = PrimaryCoords != fLeft.primary ? GetSystemMetrics(SM_CXSCREEN) : SCREEN_WIDTH;
		int center = fLeft.center ? width / 2 : 0;
		fLeft.n = (fLeft.negative ? ptNew.x - width + center : ptNew.x - center) - left;
	}
	if (ptNew.y != pt.y)
	{
		int top = PrimaryCoords != fTop.primary ? 0 : SCREEN_TOP;
		int height = PrimaryCoords != fTop.primary ? GetSystemMetrics(SM_CYSCREEN) : SCREEN_HEIGHT;
		int center = fTop.center ? height / 2 : 0;
		fTop.n = (fTop.negative ? ptNew.y - height + center : ptNew.y - center) - top;
	}
}

/*void Shortcut::onUpdatePos(Message& message)
{
	WINDOWPOS *wp = (WINDOWPOS *)message.lParam;
	HWND hwnd1, hwnd2;

	if (!fChangeZOrder)
		wp->hwndInsertAfter = GetNextWindow(hWnd, GW_HWNDPREV);
	else
		fChangeZOrder = false;

	if (!fIsTopMost)
	{
		hwnd1 = GetNextWindow(wp->hwnd, GW_HWNDPREV);
		GetClassName(hwnd1, buffer, MAX_LINE_LENGTH);
		if ((stricmp(buffer, "ShortcutClass") != 0) && (stricmp(buffer, "DesktopBackgroundClass") != 0))
		{
			// just to prevent infinite loop...
			for (int i = 0; i < 100; i++)
			{
				hwnd2 = hwnd1;
				hwnd1 = GetNextWindow(hwnd2, GW_HWNDPREV);
				GetClassName(hwnd1, buffer, MAX_LINE_LENGTH);
				if (stricmp(buffer, "ShortcutClass") == 0)
					break;
			}
			wp->hwndInsertAfter = hwnd2;
		}
	}
}*/

void Shortcut::onUpdateAlpha(Message& message)
{
	if (message.lParam < 0)
		updateAlpha(alpha.initial);
	else
		updateAlpha(message.lParam);
}

void Shortcut::updateAlpha(BYTE newAlpha, bool updatePos)
{
	if (!(alpha_enable && img1))
		return;

	alpha.current = newAlpha;
	alpha.enable = img1->has_alpha;

	if (updatePos)
	{
		WINDOWPLACEMENT win_pos;
		GetWindowPlacement(hWnd, &win_pos);
		fRelOrigin.x = win_pos.rcNormalPosition.left;
		fRelOrigin.y = win_pos.rcNormalPosition.top;
		fWidth = win_pos.rcNormalPosition.right - win_pos.rcNormalPosition.left;
		fHeight = win_pos.rcNormalPosition.bottom - win_pos.rcNormalPosition.top;
	}

	HDC dcScreen = GetDC(NULL);
	HDC dcMemory = CreateCompatibleDC(dcScreen);

	HBITMAP pOldBitmap = HBITMAP(SelectObject(dcMemory, img1->current->image));

	POINT ptWindowScreenPosition = {fRelOrigin.x, fRelOrigin.y};
	SIZE szWindow = {fWidth, fHeight};

	BLENDFUNCTION blendPixelFunction = { AC_SRC_OVER, 0, newAlpha, alpha.enable ? AC_SRC_ALPHA : 0 };
	POINT ptSrc ={0,0};

	BOOL bRet= ::UpdateLayeredWindow(hWnd, dcScreen, &ptWindowScreenPosition, &szWindow, dcMemory,
		&ptSrc, 0, &blendPixelFunction, ULW_ALPHA);

	SelectObject(dcMemory, pOldBitmap);
	DeleteDC(dcMemory);
	ReleaseDC(NULL, dcScreen);/**/
}
