/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2001 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef __SHORTCUT_H_
#define __SHORTCUT_H_

#include <map>
#include "common.h"
#include "lswinbase.h"
#include "layercut.h"
#include "execution.h"
#include "utility.h"

using namespace std;

// Alpha blending

#ifndef WS_EX_LAYERED
#define WS_EX_COMPOSITED        0x02000000L
#define WS_EX_NOACTIVATE        0x08000000L
#define WS_EX_LAYERED           0x00080000L
#define LWA_COLORKEY            0x00000001L
#define LWA_ALPHA               0x00000002L
#define ULW_ALPHA               0x00000002L
#endif // WS_EX_LAYERED

#ifndef AC_SRC_ALPHA
#define AC_SRC_ALPHA            0x00000001L
#endif

#define WM_ALPHA		(WM_USER + 0x42)

typedef BOOL (WINAPI *lpfnUpdateLayeredWindow)(HWND hwnd, HDC hdcDst, POINT *pptDst, SIZE *psize, HDC hdcSrc, POINT *pptSrc, COLORREF crKey, BLENDFUNCTION *pblend, DWORD dwFlags);
extern lpfnUpdateLayeredWindow UpdateLayeredWindow;
extern int default_alpha;
extern bool alpha_enable;
extern BYTE alpha_threshold;
extern bool loadingOnStartup;

typedef struct s_range_mono {
	int			current;
	int			initial;
	int			final;
	float		delta;
	bool		enable;
} t_range_mono;

class Shortcut;

typedef map<UINT, Shortcut*> ShortcutMap;

class CImageStorage;
class CImage;

class ShortcutFactory : public Window
{
protected:
  vector<Lsbox> vector_box;
  HWND fShortcutHints;
  ShortcutMap fShortcutGroups;
  LPSTR fSoundClick;
  LPSTR fSoundHover;
  bool fShowCaptions;
  bool fBangLock;
	CImageStorage imageStorage;
	CExecStorage execStorage;

public:
  inline LPCSTR getSoundClick() { return fSoundClick; }
  inline LPCSTR getSoundHover() { return fSoundHover; }
  inline bool getShowCaptions() { return fShowCaptions; }

  ShortcutFactory(HWND parentWnd, int& code);
  ~ShortcutFactory();

  void relayHintMessage(HWND hWnd, Message& message);
  void addHint(HWND hWnd, LPSTR caption);
  void removeHint(HWND hWnd);

	inline void loadAnimation() { imageStorage.loadAnimation(); };
	inline void loadExec() { execStorage.loadExec(); };
	inline CAnimation *loadImage(string &str, CAnimation *img) { return imageStorage.loadImage(str, img); };
	inline CAnimation *getImage(string &str, int pos) { return imageStorage.getImage(str, pos); };
	inline CExec *getExec(string &str) { return execStorage.getExec(str); };

  void ModuleHook(HWND caller, LPCSTR args);
protected:
  void createShortcuts(Lsbox *box = NULL);
  void destroyShortcuts();

  void setVisible(LPCSTR groups, int state);
  void setTopMost(LPCSTR groups, int topMost);
	void setAlpha(LPCSTR groups);

  static void bangGroupToggle(HWND sender, LPCSTR args);
  static void bangGroupShow(HWND sender, LPCSTR args);
  static void bangGroupHide(HWND sender, LPCSTR args);
  static void bangGroupOnTop(HWND sender, LPCSTR args);
  static void bangGroupOnBottom(HWND sender, LPCSTR args);
  static void bangGroupOnTopToggle(HWND sender, LPCSTR args);
  static void bangGroupSetAlpha(HWND sender, LPCSTR args);

  virtual void windowProc(Message& message);
  void onCreate(Message& message);
  void onDestroy(Message& message);
  void onGetRevId(Message& message);
  void onRefresh(Message& message);
  void onSetTopmost(Message& message);
};


enum ShortcutState {ssNormal, ssHover, ssClick, ssNormalAfterClicked};

class Shortcut : public Window
{
protected:
  ShortcutFactory* fOwner;
  LPSTR fCaption;
  LPSTR fId;
  LPSTR fCommand;
  LPSTR fCommandArgs;
  LPSTR fCommand2;
  LPSTR fCommand2Args;
  LPSTR fHoverSound;
  LPSTR fClickSound;
	CExec *fExec;
  string fNormalName;
  string fHoverName;
  string fClickName;
	CAnimation *fNormalImage;
	CAnimation *fHoverImage;
	CAnimation *fClickImage;
	CAnimation *nextAnim;
	CAnimation *img1;
	CAnimation *img2;
  Lsbox *m_box;
  POINT fRelOrigin;
  coord fLeft;
  coord fTop;
  int fWidth;
  int fHeight;
  ShortcutState fState;
  Shortcut* fNext;
//  bool fChangeZOrder;
  bool fIsVisible;
  bool fIsTopMost;
  bool fIsOpaque;
  bool fIsTransparent;
  bool fShowCaption;
	bool fExecHover;
	bool fExecOut;
  UINT fLeaveTimer;
	UINT fFadeTimer;
	UINT fAnimTimer;
	int countAnim;
	int lastPos;
  HDC fPaintDC;
	t_range_mono alpha;

public:
  Shortcut(ShortcutFactory* owner, LPCSTR szConfigLine, DWORD& group, Lsbox *box = NULL);
  ~Shortcut();

  void append(Shortcut* shortcut);
  void setVisible(int visible);
  void setTopMost(int topMost);
	void setAlpha(int val);

protected:
  DWORD parseConfig(LPCSTR szLine);
  void createShortcut();
  void clear();
  void setImage(CAnimation *anim, bool reset = false);
  void updateAlpha(BYTE newAlpha= 255, bool updatePos = true);
	void timerHover();
	void timerFade();
	void timerAnim();

  virtual void windowProc(Message& message);
  void onCreate(Message& message);
  void onDestroy(Message& message);
  void onEndSession(Message& message);
  void onSysCommand(Message& message);
  void onMouseMove(Message& message);
	void onMouseLeave(Message& message);
  void onMouseActivate(Message& message);
  void onMove(Message& message);
  void onTimer(Message& message);
  void onButtonDown(Message& message);
  void onLButtonUp(Message& message);
  void onRButtonUp(Message& message);
  void onMButtonUp(Message& message);
  void onDropFiles(Message& message);
  void onEraseBkgnd(Message& message);
  void onDisplayChange(Message& message);
  void onShowWindow(Message& message);

  void onUpdateAlpha(Message& message);
//	void onUpdatePos(Message& message);
};

void LsboxModuleHook(HWND caller, const char *bangCommandName, const char *arguments);

HRGN BitmapToRegionAlpha(HBITMAP hbm, BYTE clrTransp, int xoffset, int yoffset);
void PreMultiplyRGBChannels(HBITMAP &bmp, LPBYTE pBitmapBits);

extern "C"
{
  __declspec( dllexport ) int initModuleEx(HWND parent, HINSTANCE dll, LPCSTR szPath);
  __declspec( dllexport ) void quitModule(HINSTANCE dllInst);
}

#endif

