/*

  This is a part of Layercut.dll source code.
  
  Copyright (C) 2001-2002 Sylvain Rouquette
  rouque_s@epita.fr
  http://www.epita.fr:8000/~rouque_s

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "execution.h"
#include "debug.h"
#include "safestr.h"

using namespace std;

/**************************************************************************************/

CExecStorage::CExecStorage()
{}

CExecStorage::~CExecStorage()
{
	for (int i = 0; i < list.size(); i++)
		delete list[i];
}

/**************************************************************************************/

void CExecStorage::loadExec()
{
	char configLine[MAX_LINE_LENGTH],
		 token1[MAX_LINE_LENGTH],
		 token2[MAX_LINE_LENGTH],
		 token3[MAX_LINE_LENGTH],
		 extra[MAX_LINE_LENGTH];
	FILE *f;
	char *tokens[] = {token1, token2, token3};
	string name;
	CExec *p_ex;
	int ret;
	bool is_exec = false;

	f = LCOpen(NULL);
	if (!f)
		return;

	while(LCReadNextConfig(f, "*Layercut", configLine, MAX_LINE_LENGTH))
	{
		ret = LCTokenize (configLine, tokens, 3, extra);
		if (stricmp(token2, "execution") == 0)
		{
			is_exec = true;
			p_ex = new CExec();
			p_ex->name = string(token3);
		}
		if (is_exec)
		{
			if (stricmp(token2, "left") == 0)
			{
				p_ex->left = string(token3);
				p_ex->left_arg = string(extra);
				p_ex->b_left = true;
			}
			else if (stricmp(token2, "right") == 0)
			{
				p_ex->right = string(token3);
				p_ex->right_arg = string(extra);
				p_ex->b_right = true;
			}
			else if (stricmp(token2, "middle") == 0)
			{
				p_ex->middle = string(token3);
				p_ex->middle_arg = string(extra);
				p_ex->b_middle = true;
			}
			else if (stricmp(token2, "drop") == 0)
			{
				p_ex->drop = string(token3);
				p_ex->drop_arg = string(extra);
				p_ex->b_drop = true;
			}
			else if (stricmp(token2, "in") == 0)
			{
				p_ex->in = string(token3);
				p_ex->in_arg = string(extra);
				p_ex->b_in = true;
			}
			else if (stricmp(token2, "out") == 0)
			{
				p_ex->out = string(token3);
				p_ex->out_arg = string(extra);
				p_ex->b_out = true;
			}
			else if (stricmp(token2, "~execution") == 0)
			{
					list.push_back(p_ex);
					qsort(0, list.size() - 1);
				is_exec = false;
			}
		}
	}
	LCClose(f);
}

/**************************************************************************************/

void CExecStorage::bubble(CExec *exec, int deb, int fin)
{
	int ret;
	int milieu;

	while (fin - deb > 1)
	{
		milieu = (deb + fin) / 2;
		ret = list[milieu]->name.compare(exec->name);
		if (ret > 0)
			fin = milieu;
		else if (ret < 0)
			deb = milieu;
		else
			break;
	}
	list.insert(list.begin() + deb, exec);
}

/**************************************************************************************/

void CExecStorage::qsort(int deb, int fin)
{
	int i, j;
	string *milieu;
	CExec *temp;

	i = deb;
	j = fin;
	milieu = &(list[(deb + fin) / 2]->name);

	while (i <= j)
	{
		while(list[i]->name.compare(*milieu) < 0) i++;
		while(list[j]->name.compare(*milieu) > 0) j--;

		if(i <= j)
		{
			temp = list[i];
			list[i] = list[j];
			list[j] = temp;
			i++; j--;
		}
	}

	if (i < fin) qsort(i, fin);
	if (deb < j) qsort(deb, j);
}

/**************************************************************************************/

CExec *CExecStorage::getExec(string &str)
{
	int milieu = 0;
	int ret = -1;
	int deb = 0, fin = list.size() - 1;

	if (list.empty() || str.empty())
		return NULL;

	do
	{
		milieu = (deb + fin) / 2;
		ret = list[milieu]->name.compare(str);
		if (ret > 0)
			fin = milieu - 1;
		else if (ret < 0)
			deb = milieu + 1;
		else
			break;
	} while (fin - deb >= 0);
	/**/
	/*for (milieu = 0; milieu < list.size(); milieu++)
	{
		ret = list[milieu]->name.compare(str);
		if (ret == 0)
			break;
	}/**/

	if (ret == 0)
		return (list[milieu]);
	else
		return NULL;
}
