/*

  This is a part of Layercut.dll source code.
  
  Copyright (C) 2001-2002 Sylvain Rouquette
  rouque_s@epita.fr
  http://www.epita.fr:8000/~rouque_s

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "utility.h"

extern bool PrimaryCoords;
extern bool BoundToMonitor;


void ModifyStyle(IN HWND hWnd, IN DWORD removeStyle, IN DWORD addStyle)
{
	SetWindowLong(hWnd, GWL_STYLE,
		(GetWindowLong(hWnd, GWL_STYLE) & ~removeStyle) | addStyle);
}


HWND GetLitestepDesktop()
{
	// try to find the window of the desktop module
	HWND hWnd = FindWindow("DesktopBackgroundClass", NULL);
	
	if (!IsWindow(hWnd))
	{
		// not found, get windows' desktop
		hWnd = FindWindow("ProgMan", "Program Manager");
		hWnd = FindWindowEx(hWnd, NULL, "SHELLDLL_DefView", NULL);
		HWND hwnd = FindWindowEx(hWnd, NULL, "SysListView32", NULL);
		if ( (GetWindowLong(hwnd, GWL_STYLE) & WS_VISIBLE) )
			hWnd = hwnd;
	}
	
	return hWnd;
}

void zOrder(HWND hWnd, Lsbox *box, bool bTopMost)
{
	if (!box)
	{
		ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
		SetParent(hWnd, bTopMost ? NULL : GetLitestepDesktop());
		ModifyStyle(hWnd, WS_CHILD, WS_POPUP);
	}

	SetWindowPos(hWnd, bTopMost ? HWND_TOPMOST : HWND_BOTTOM,
		0, 0, 0, 0,
		SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
}


long convertCoord(const coord &pos, int origin, int maxVal)
{
	long value = pos.n;

	if (pos.center)
		value += (maxVal / 2);
	else if (!pos.absolute && (value < 0 || (value == 0 && pos.negative) ))
		value += maxVal;

	return origin + value;
}

POINT convertCoords(const coord &left, const coord &top)
{
	POINT pt;

	if (PrimaryCoords != left.primary)
		pt.x = convertCoord(left, 0, GetSystemMetrics(SM_CXSCREEN));
	else
		pt.x = convertCoord(left, SCREEN_LEFT, SCREEN_WIDTH);

	if (PrimaryCoords != top.primary)
		pt.y = convertCoord(top, 0, GetSystemMetrics(SM_CYSCREEN));
	else
		pt.y = convertCoord(top, SCREEN_TOP, SCREEN_HEIGHT);


	if (BoundToMonitor && !MonitorFromPoint(pt, MONITOR_DEFAULTTONULL))
	{
		RECT rcMon;
		MONITORINFO mi;
		mi.cbSize = sizeof(mi);
		
		GetMonitorInfo(MonitorFromPoint(pt, MONITOR_DEFAULTTONEAREST), &mi);
		rcMon = mi.rcMonitor;
		
		if (pt.x < rcMon.left)
			pt.x = rcMon.left;
		else if (pt.x > rcMon.right)
			pt.x = rcMon.right;
		
		if (pt.y < rcMon.top)
			pt.y = rcMon.top;
		else if (pt.y > rcMon.bottom)
			pt.y = rcMon.bottom;
	}
	
	return pt;
}
